@@ -1,25 +1,25 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::ChildManager;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
+our $VERSION = '0.51';
 
 # Not a notifier
 
 use IO::Async::Stream;
-use IO::Async::MergePoint;
+
+use IO::Async::OS;
 
 use Carp;
 use Scalar::Util qw( weaken );
 
-use Fcntl qw( F_GETFL F_SETFL FD_CLOEXEC );
-use POSIX qw( _exit sysconf _SC_OPEN_MAX dup2 nice );
+use POSIX qw( _exit sysconf _SC_OPEN_MAX dup dup2 nice );
 
 use constant LENGTH_OF_I => length( pack( "I", 0 ) );
 
@@ -38,7 +38,7 @@ This object is used indirectly via an C<IO::Async::Loop>:
  use IO::Async::Loop;
  use POSIX qw( WEXITSTATUS );
 
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  ...
 
@@ -57,10 +57,9 @@ This object is used indirectly via an C<IO::Async::Loop>:
 
     stdout => {
        on_read => sub {
-          my ( $stream, $buffref, $closed ) = @_;
-          if( $$buffref =~ s/^(.*)\n// ) {
+          my ( $stream, $buffref, $eof ) = @_;
+          while( $$buffref =~ s/^(.*)\n// ) {
              print "PING wrote: $1\n";
-             return 1;
           }
           return 0;
        },
@@ -73,7 +72,7 @@ This object is used indirectly via an C<IO::Async::Loop>:
     },
  );
 
- my ( $pipeRd, $pipeWr ) = $loop->pipepair;
+ my ( $pipeRd, $pipeWr ) = IO::Async::OS->pipepair;
  $loop->spawn_child(
     command => "/usr/bin/my-command",
 
@@ -93,7 +92,7 @@ This object is used indirectly via an C<IO::Async::Loop>:
 
  $loop->spawn_child(
     code => sub {
-       do_something(); # executes in a child process
+       do_something; # executes in a child process
        return 1;
     },
 
@@ -107,7 +106,7 @@ This object is used indirectly via an C<IO::Async::Loop>:
 
 =head1 DESCRIPTION
 
-This module extends the functionallity of the containing C<IO::Async::Loop> to
+This module extends the functionality of the containing C<IO::Async::Loop> to
 manage the execution of child processes. It acts as a central point to store
 PID values of currently-running children, and to call the appropriate
 continuation handler code when the process terminates. It provides useful
@@ -116,6 +115,54 @@ to capture the child process's STDOUT and STDERR streams.
 
 =cut
 
+# Writing to variables of $> and $) have tricky ways to obtain error results
+sub setuid
+{
+   my ( $uid ) = @_;
+
+   $> = $uid; my $saved_errno = $!;
+   $> == $uid and return 1;
+
+   $! = $saved_errno;
+   return undef;
+}
+
+sub setgid
+{
+   my ( $gid ) = @_;
+
+   $) = $gid; my $saved_errno = $!;
+   $) == $gid and return 1;
+
+   $! = $saved_errno;
+   return undef;
+}
+
+sub setgroups
+{
+   my @groups = @_;
+
+   my $gid = $)+0;
+   # Put the primary GID as the first group in the supplementary list, because
+   # some operating systems ignore this position, expecting it to indeed be
+   # the primary GID.
+   # See
+   #   https://rt.cpan.org/Ticket/Display.html?id=65127
+   @groups = grep { $_ != $gid } @groups;
+
+   $) = "$gid $gid " . join " ", @groups; my $saved_errno = $!;
+
+   # No easy way to detect success or failure. Just check that we have all and
+   # only the right groups
+   my %gotgroups = map { $_ => 1 } split ' ', "$)";
+
+   $! = $saved_errno;
+   $gotgroups{$_}-- or return undef for @groups;
+   keys %gotgroups or return undef;
+
+   return 1;
+}
+
 # Internal constructor
 sub new
 {
@@ -140,75 +187,6 @@ object.
 
 =cut
 
-=head2 $pid = $loop->detach_child( %params )
-
-This method creates a new child process to run a given code block.
-
-=over 8
-
-=item code => CODE
-
-A block of code to execute in the child process. It will be called in scalar
-context inside an C<eval> block. The return value will be used as the
-C<exit()> code from the child if it returns (or 255 if it returned C<undef> or
-thows an exception).
-
-=item on_exit => CODE
-
-A optional continuation to be called when the child processes exits. It will
-be invoked in the following way:
-
- $on_exit->( $pid, $exitcode )
-
-The second argument is passed the plain perl C<$?> value. To use that
-usefully, see C<WEXITSTATUS()> and others from C<POSIX>.
-
-This key is optional; if not supplied, the calling code should install a
-handler using the C<watch_child()> method.
-
-=item keep_signals => BOOL
-
-Optional boolean. If missing or false, any CODE references in the C<%SIG> hash
-will be removed and restored back to C<DEFAULT> in the child process. If true,
-no adjustment of the C<%SIG> hash will be performed.
-
-=back
-
-=cut
-
-sub detach_child
-{
-   my $self = shift;
-   my %params = @_;
-
-   my $code = $params{code};
-
-   my $kid = fork();
-   defined $kid or croak "Cannot fork() - $!";
-
-   if( $kid == 0 ) {
-      unless( $params{keep_signals} ) {
-         foreach( keys %SIG ) {
-            next if m/^__(WARN|DIE)__$/;
-            $SIG{$_} = "DEFAULT" if ref $SIG{$_} eq "CODE";
-         }
-      }
-
-      my $exitvalue = eval { $code->() };
-
-      defined $exitvalue or $exitvalue = -1;
-      _exit( $exitvalue );
-   }
-
-   my $loop = $self->{loop};
-
-   if( defined $params{on_exit} ) {
-      $loop->watch_child( $kid => $params{on_exit} );
-   }
-
-   return $kid;
-}
-
 =head2 $pid = $loop->spawn_child( %params )
 
 This method creates a new child process to run a given code block or command.
@@ -220,7 +198,7 @@ The C<%params> hash takes the following keys:
 
 Either a reference to an array containing the command and its arguments, or a
 plain string containing the command. This value is passed into perl's
-C<exec()> function.
+C<exec> function.
 
 =item code => CODE
 
@@ -240,25 +218,25 @@ in the following way:
  $on_exit->( $pid, $exitcode, $dollarbang, $dollarat )
 
 The second argument is passed the plain perl C<$?> value. To use that
-usefully, see C<WEXITSTATUS()> and others from C<POSIX>.
+usefully, see C<WEXITSTATUS> and others from C<POSIX>.
 
 =back
 
 Exactly one of the C<command> or C<code> keys must be specified.
 
 If the C<command> key is used, the given array or string is executed using the
-C<exec()> function. 
+C<exec> function. 
 
-If the C<code> key is used, the return value will be used as the C<exit()>
+If the C<code> key is used, the return value will be used as the C<exit(2)>
 code from the child if it returns (or 255 if it returned C<undef> or thows an
 exception).
 
- Case            | WEXITSTATUS($exitcode) | $dollarbang | $dollarat
- ----------------+------------------------+-------------+----------
- exec() succeeds | exit code from program |     0       |    ""
- exec() fails    |         255            |     $!      |    ""
- $code returns   |     return value       |     $!      |    ""
- $code dies      |         255            |     $!      |    $@
+ Case          | WEXITSTATUS($exitcode) | $dollarbang | $dollarat
+ --------------+------------------------+-------------+----------
+ exec succeeds | exit code from program |     0       |    ""
+ exec fails    |         255            |     $!      |    ""
+ $code returns |     return value       |     $!      |    ""
+ $code dies    |         255            |     $!      |    $@
 
 It is usually more convenient to use the C<open_child> method in simple cases
 where an external program is being started in order to interact with it via
@@ -291,12 +269,15 @@ sub spawn_child
 
    my $loop = $self->{loop};
 
-   my ( $readpipe, $writepipe ) = $loop->pipepair() or croak "Cannot pipe() - $!";
+   my ( $readpipe, $writepipe );
 
-   my $flags = fcntl( $writepipe, F_GETFL, 0 ) or 
-      croak "Cannot fcntl(F_GETFL) - $!";
-   fcntl( $writepipe, F_SETFL, $flags | FD_CLOEXEC ) or
-      croak "Cannot fcntl(F_SETFL) - $!";
+   {
+      # Ensure it's FD_CLOEXEC - this is a bit more portable than manually
+      # fiddling with F_GETFL and F_SETFL (e.g. MSWin32)
+      local $^F = -1;
+
+      ( $readpipe, $writepipe ) = IO::Async::OS->pipepair or croak "Cannot pipe() - $!";
+   }
 
    if( defined $command ) {
       my @command = ref( $command ) ? @$command : ( $command );
@@ -308,7 +289,7 @@ sub spawn_child
       };
    }
 
-   my $kid = $loop->detach_child( 
+   my $kid = $loop->fork( 
       code => sub {
          # Child
          close( $readpipe );
@@ -324,7 +305,7 @@ sub spawn_child
 =head2 C<setup> array
 
 This array gives a list of file descriptor operations to perform in the child
-process after it has been C<fork()>ed from the parent, before running the code
+process after it has been C<fork(2)>ed from the parent, before running the code
 or command. It consists of name/value pairs which are ordered; the operations
 are performed in the order given.
 
@@ -343,12 +324,12 @@ The file descriptor will be closed.
 
 =item [ 'dup', $io ]
 
-The file descriptor will be C<dup2()>ed from the given IO handle.
+The file descriptor will be C<dup2(2)>ed from the given IO handle.
 
 =item [ 'open', $mode, $file ]
 
 The file descriptor will be opened from the named file in the given mode. The
-C<$mode> string should be in the form usually given to the C<open()> function;
+C<$mode> string should be in the form usually given to the C<open> function;
 such as '<' or '>>'.
 
 =item [ 'keep' ]
@@ -387,11 +368,11 @@ A reference to a hash to set as the child process's environment.
 
 =item nice => INT
 
-Change the child process's scheduling priority using C<POSIX::nice()>.
+Change the child process's scheduling priority using C<POSIX::nice>.
 
 =item chdir => STRING
 
-Change the child process's working directory using C<chdir()>.
+Change the child process's working directory using C<chdir>.
 
 =item setuid => INT
 
@@ -409,7 +390,8 @@ C<IO::Async> will B<NOT> check before detaching the child process whether
 this is the case.
 
 If setting both the primary GID and the supplementary groups list, it is
-suggested to set the primary GID first.
+suggested to set the primary GID first. Moreover, some operating systems may
+require that the supplementary groups list contains the primary GID.
 
 =back
 
@@ -509,7 +491,7 @@ sub _spawn_in_parent
 
    my $loop = $self->{loop};
 
-   # We need to wait for both the errno pipe to close, and for waitpid()
+   # We need to wait for both the errno pipe to close, and for waitpid
    # to give us an exit code. We'll form two closures over these two
    # variables so we can cope with those happening in either order
 
@@ -519,10 +501,12 @@ sub _spawn_in_parent
    my $pipeclosed = 0;
 
    $loop->add( IO::Async::Stream->new(
+      notifier_name => "statuspipe,kid=$kid",
+
       read_handle => $readpipe,
 
       on_read => sub {
-         my ( $self, $buffref, $closed ) = @_;
+         my ( $self, $buffref, $eof ) = @_;
 
          if( !defined $dollarbang ) {
             if( length( $$buffref ) >= 2 * LENGTH_OF_I ) {
@@ -538,7 +522,7 @@ sub _spawn_in_parent
             }
          }
 
-         if( $closed ) {
+         if( $eof ) {
             $dollarbang = 0  if !defined $dollarbang;
             if( !defined $length_dollarat ) {
                $length_dollarat = 0;
@@ -581,12 +565,15 @@ sub _spawn_in_child
       # Count of how many times we'll need to use the current handles.
       my %fds_refcount = %fd_in_use;
 
+      # To dup2() without clashes we might need to temporarily move some handles
+      my %dup_from;
+
       my $max_fd = 0;
       my $writepipe_clashes = 0;
 
       if( @$setup ) {
          # The writepipe might be in the way of a setup filedescriptor. If it
-         # is we'll have to dup2() it out of the way then close the original.
+         # is we'll have to dup2 it out of the way then close the original.
          foreach my $i ( 0 .. $#$setup/2 ) {
             my ( $key, $value ) = @$setup[$i*2, $i*2 + 1];
             $key =~ m/^fd(\d+)$/ or next;
@@ -606,9 +593,11 @@ sub _spawn_in_child
                $fd_in_use{$fd} = 1;
 
                my $fileno = fileno $params[0];
-               # Keep a count of how many times it will be dup()ed from so we
+               # Keep a count of how many times it will be dup'ed from so we
                # can close it once we've finished
                $fds_refcount{$fileno}++;
+
+               $dup_from{$fileno} = $fileno;
             };
 
             $operation eq "keep" and do {
@@ -643,13 +632,20 @@ sub _spawn_in_child
                   my $from = fileno $params[0];
 
                   if( $from != $fd ) {
+                     if( exists $dup_from{$fd} ) {
+                        defined( $dup_from{$fd} = dup( $fd ) ) or die "Cannot dup($fd) - $!";
+                     }
+
+                     my $real_from = $dup_from{$from};
+
                      POSIX::close( $fd );
-                     dup2( $from, $fd ) or die "Cannot dup2($from to $fd) - $!\n";
+                     dup2( $real_from, $fd ) or die "Cannot dup2($real_from to $fd) - $!\n";
                   }
 
                   $fds_refcount{$from}--;
                   if( !$fds_refcount{$from} and !$fd_in_use{$from} ) {
                      POSIX::close( $from );
+                     delete $dup_from{$from};
                   }
                };
 
@@ -673,15 +669,13 @@ sub _spawn_in_child
                chdir( $value ) or die "Cannot chdir('$value') - $!";
             }
             elsif( $key eq "setuid" ) {
-               $> = $value or die "Cannot setuid('$value') - $!";
+               setuid( $value ) or die "Cannot setuid('$value') - $!";
             }
             elsif( $key eq "setgid" ) {
-               $) = $value or die "Cannot setgid('$value') - $!";
+               setgid( $value ) or die "Cannot setgid('$value') - $!";
             }
             elsif( $key eq "setgroups" ) {
-               my $gid = $)+0;
-               my $groups = join( " ", @$value );
-               $) = "$gid $groups" or die "Cannot setgroups('$groups') - $!";
+               setgroups( @$value ) or die "Cannot setgroups() - $!";
             }
          }
       }
@@ -698,344 +692,10 @@ sub _spawn_in_child
    return $exitvalue;
 }
 
-=head2 $pid = $loop->open_child( %params )
-
-This creates a new child process to run the given code block or command, and
-attaches filehandles to it that the parent will watch. The C<%params> hash
-takes the following keys:
-
-=over 8
-
-=item command => ARRAY or STRING
-
-=item code => CODE
-
-The command or code to run in the child process (as per the C<spawn> method)
-
-=item on_finish => CODE
-
-A continuation to be called when the child process exits and has closed all of
-the filehandles that were set up for it. It will be invoked in the following
-way:
-
- $on_finish->( $pid, $exitcode )
-
-The second argument is passed the plain perl C<$?> value. To use that
-usefully, see C<WEXITSTATUS()> and others from C<POSIX>.
-
-=item on_error => CODE
-
-Optional continuation to be called when the child code block throws an
-exception, or the command could not be C<exec()>ed. It will be invoked in the
-following way (as per C<spawn>)
-
- $on_error->( $pid, $exitcode, $dollarbang, $dollarat )
-
-If this continuation is not supplied, then C<on_finish> is used instead. The
-value of C<$!> and C<$@> will not be reported.
-
-=item setup => ARRAY
-
-Optional reference to an array to pass to the underlying C<spawn> method.
-
-=back
-
-In addition, the hash takes keys that define how to set up file descriptors in
-the child process. (If the C<setup> array is also given, these operations will
-be performed after those specified by C<setup>.)
-
-=over 8
-
-=item fdI<n> => HASH
-
-A hash describing how to set up file descriptor I<n>. The hash may contain one
-of the following sets of keys:
-
-=over 4
-
-=item on_read => CODE
-
-The child will be given the writing end of a pipe. The reading end will be
-wrapped by an C<IO::Async::Stream> using this C<on_read> callback function.
-
-=item from => STRING
-
-The child will be given the reading end of a pipe. The string given by the
-C<from> parameter will be written to the child. When all of the data has been
-written the pipe will be closed.
-
-=back
-
-=item stdin => ...
-
-=item stdout => ...
-
-=item stderr => ...
-
-Shortcuts for C<fd0>, C<fd1> and C<fd2> respectively.
-
-=back
-
-=cut
-
-sub open_child
-{
-   my $self = shift;
-   my %params = @_;
-
-   my %subparams;
-   my @setup;
-   my %filehandles;
-
-   my $on_finish = delete $params{on_finish};
-   ref $on_finish or croak "Expected 'on_finish' to be a reference";
-
-   my $on_error = delete $params{on_error};
-   if( $on_error ) {
-      ref $on_error or croak "Expected 'on_error' to be a reference";
-   }
-
-   $params{on_exit} and croak "Cannot pass 'on_exit' parameter through ChildManager->open";
-
-   if( $params{setup} ) {
-      ref $params{setup} eq "ARRAY" or croak "Expected 'setup' to be an ARRAY ref";
-      @setup = @{ $params{setup} };
-      delete $params{setup};
-   }
-
-   my $loop = $self->{loop};
-
-   foreach my $key ( keys %params ) {
-      my $value = $params{$key};
-
-      my $orig_key = $key;
-
-      # Rewrite stdin/stdout/stderr
-      $key eq "stdin"  and $key = "fd0";
-      $key eq "stdout" and $key = "fd1";
-      $key eq "stderr" and $key = "fd2";
-
-      if( $key =~  m/^fd\d+$/ ) {
-         ref $value eq "HASH" or croak "Expected '$orig_key' to be a HASH ref";
-
-         my $op;
-
-         if( exists $value->{on_read} ) {
-            ref $value->{on_read} or croak "Expected 'on_read' for '$orig_key' to be a reference";
-            scalar keys %$value == 1 or croak "Found other keys than 'on_read' for '$orig_key'";
-
-            $op = "pipe_read";
-         }
-         elsif( exists $value->{from} ) {
-            ref $value->{from} eq "" or croak "Expected 'from' for '$orig_key' not to be a reference";
-            scalar keys %$value == 1 or croak "Found other keys than 'from' for '$orig_key'";
-
-            $op = "pipe_write";
-         }
-         else {
-            croak "Cannot recognise what to do with '$orig_key'";
-         }
-
-         my ( $myfd, $childfd );
-
-         if( $op eq "pipe_read" ) {
-            ( $myfd, $childfd ) = $loop->pipepair() or croak "Unable to pipe() - $!";
-         }
-         elsif( $op eq "pipe_write" ) {
-            ( $childfd, $myfd ) = $loop->pipepair() or croak "Unable to pipe() - $!";
-         }
-
-         $filehandles{$key} = [ $myfd, $childfd, $value ];
-         push @setup, $key => [ dup => $childfd ];
-      }
-      else {
-         $subparams{$orig_key} = $value;
-      }
-   }
-
-   my $pid;
-
-   my $mergepoint = IO::Async::MergePoint->new(
-      needs => [ "exit", keys %filehandles ],
-
-      on_finished => sub {
-         my %items = @_;
-         my ( $exitcode, $dollarbang, $dollarat ) = @{ $items{exit} };
-
-         if( $params{code} and $dollarat eq "" or $params{command} and $dollarbang == 0 ) {
-            $on_finish->( $pid, $exitcode );
-         }
-         else {
-            if( $on_error ) {
-               $on_error->( $pid, $exitcode, $dollarbang, $dollarat );
-            }
-            else {
-               $on_finish->( $pid, $exitcode ); # Don't have a way to report dollarbang/dollarat
-            }
-         }
-      },
-   );
-
-   $pid = $loop->spawn_child( %subparams, 
-      setup => \@setup,
-      on_exit => sub {
-         my ( undef, $exitcode, $dollarbang, $dollarat ) = @_;
-         $mergepoint->done( "exit", [ $exitcode, $dollarbang, $dollarat ] );
-      },
-   );
-
-   return undef unless defined $pid;
-
-   # Now install the handlers
-
-   foreach my $fd ( keys %filehandles ) {
-      my ( $myfd, $childfd, $fdopts ) = @{ $filehandles{$fd} };
-
-      close( $childfd );
-
-      my $notifier;
-
-      if( exists $fdopts->{on_read} ) {
-         my $on_read = $fdopts->{on_read};
-
-         $notifier = IO::Async::Stream->new(
-            read_handle => $myfd,
-
-            on_read => $on_read,
-
-            on_closed => sub {
-               $mergepoint->done( $fd );
-            },
-         );
-      }
-      elsif( exists $fdopts->{from} ) {
-         $notifier = IO::Async::Stream->new(
-            write_handle => $myfd,
-
-            on_outgoing_empty => sub {
-               $notifier->close;
-            },
-
-            on_closed => sub {
-               $mergepoint->done( $fd );
-            },
-         );
-
-         $notifier->write( $fdopts->{from} );
-      }
-
-      $loop->add( $notifier );
-   }
-
-   return $pid;
-}
-
-=head2 $pid = $loop->run_child( %params )
-
-This creates a new child process to run the given code block or command,
-capturing its STDOUT and STDERR streams. When the process exits, a
-continuation is invoked being passed the exitcode, and content of the streams.
-
-=over 8
-
-=item command => ARRAY or STRING
-
-=item code => CODE
-
-The command or code to run in the child process (as per the C<spawn> method)
-
-=item on_finish => CODE
-
-A continuation to be called when the child process exits and closed its STDOUT
-and STDERR streams. It will be invoked in the following way:
-
- $on_finish->( $pid, $exitcode, $stdout, $stderr )
-
-The second argument is passed the plain perl C<$?> value. To use that
-usefully, see C<WEXITSTATUS()> and others from C<POSIX>.
-
-=item stdin => STRING
-
-Optional. String to pass in to the child process's STDIN stream.
-
-=item setup => ARRAY
-
-Optional reference to an array to pass to the underlying C<spawn> method.
-
-=back
-
-This method is intended mainly as an IO::Async-compatible replacement for the
-perl C<readpipe> function (`backticks`), allowing it to replace
-
-  my $output = `command here`;
-
-with
-
- $loop->run_child(
-    command => "command here", 
-    on_finish => sub {
-       my ( undef, $exitcode, $output ) = @_;
-       ...
-    }
- );
-
-=cut
-
-sub run_child
-{
-   my $self = shift;
-   my %params = @_;
-
-   my $on_finish = delete $params{on_finish};
-   ref $on_finish or croak "Expected 'on_finish' to be a reference";
-
-   my $child_out;
-   my $child_err;
-
-   my %subparams;
-
-   if( my $child_stdin = delete $params{stdin} ) {
-      ref $child_stdin and croak "Expected 'stdin' not to be a reference";
-      $subparams{stdin} = { from => $child_stdin };
-   }
-
-   $subparams{code}    = delete $params{code};
-   $subparams{command} = delete $params{command};
-   $subparams{setup}   = delete $params{setup};
-
-   croak "Unrecognised parameters " . join( ", ", keys %params ) if keys %params;
-
-   my $loop = $self->{loop};
-   $loop->open_child(
-      %subparams,
-      stdout => {
-         on_read => sub { 
-            my ( $stream, $buffref, $closed ) = @_;
-            $child_out = $$buffref if $closed;
-            return 0;
-         }
-      },
-
-      stderr => { 
-         on_read => sub {
-            my ( $stream, $buffref, $closed ) = @_;
-            $child_err = $$buffref if $closed;
-            return 0;
-         }
-      },
-
-      on_finish => sub {
-         my ( $kid, $exitcode ) = @_;
-         $on_finish->( $kid, $exitcode, $child_out, $child_err );
-      },
-   );
-}
-
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
