@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Handle;
 
@@ -9,10 +9,12 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.30';
+our $VERSION = '0.51';
 
 use Carp;
 
+use IO::Handle; # give methods to bare IO handles
+
 =head1 NAME
 
 C<IO::Async::Handle> - event callbacks for a non-blocking file descriptor
@@ -22,13 +24,13 @@ C<IO::Async::Handle> - event callbacks for a non-blocking file descriptor
 This class is likely not to be used directly, because subclasses of it exist
 to handle more specific cases. Here is an example of how it would be used to
 watch a listening socket for new connections. In real code, it is likely that
-the C<< Loop->listen() >> method would be used instead.
+the C<< Loop->listen >> method would be used instead.
 
  use IO::Socket::INET;
  use IO::Async::Handle;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  my $socket = IO::Socket::INET->new( LocalPort => 1234, Listen => 1 );
 
@@ -36,7 +38,7 @@ the C<< Loop->listen() >> method would be used instead.
     handle => $socket,
 
     on_read_ready  => sub {
-       my $new_client = $socket->accept(); 
+       my $new_client = $socket->accept; 
        ...
     },
  );
@@ -52,30 +54,28 @@ non-stream sockets, see L<IO::Async::Socket>.
 This subclass of L<IO::Async::Notifier> allows non-blocking IO on filehandles.
 It provides event handlers for when the filehandle is read- or write-ready.
 
-This object may be used in one of two ways; as an instance with CODE
-references as callbacks, or as a base class with overridden methods.
+=cut
 
-=over 4
+=head1 EVENTS
 
-=item Subclassing
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
-If a subclass is built, then it can override the following methods to handle
-events:
+=head2 on_read_ready
 
- $self->on_read_ready()
+Invoked when the read handle becomes ready for reading.
 
- $self->on_write_ready()
+=head2 on_write_ready
 
-Optionally, an C<on_closed> method may be provided, which will be called when
-the C<close> method is invoked.
+Invoked when the write handle becomes ready for writing.
 
- $self->on_closed()
+=head2 on_closed
 
-=back
+Optional. Invoked when the handle becomes closed.
 
-If either of the readyness methods calls the C<close()> method, then
-the handle is internally marked as closed within the object and will be
-removed from its containing loop, if it is within one.
+This handler is invoked before the filehandles are closed and the Handle
+removed from its containing Loop. The C<loop> will still return the containing
+Loop object.
 
 =cut
 
@@ -102,23 +102,16 @@ as above. Must implement C<fileno> method in way that C<IO::Handle> does.
 
 =item on_write_ready => CODE
 
-CODE references to callbacks for when the handle becomes read-ready or
-write-ready. If these are not supplied, subclass methods will be called
-instead.
-
- $on_read_ready->( $self )
-
- $on_write_ready->( $self )
-
 =item on_closed => CODE
 
-Optional. CODE reference to the callback for when the handle becomes closed.
+CODE references for event handlers.
 
-This callback is invoked before the filehandles are closed and the Handle
-removed from its containing Loop. The C<get_loop> will still return the
-containing Loop object.
+=item want_readready => BOOL
 
- $on_closed->( $self )
+=item want_writeready => BOOL
+
+If present, enable or disable read- or write-ready notification as per the
+C<want_readready> and C<want_writeready> methods.
 
 =back
 
@@ -132,7 +125,7 @@ If no IO handles are provided at construction time, the object is still
 created but will not yet be fully-functional as a Handle. IO handles can be
 assigned later using the C<set_handle> or C<set_handles> methods, or by
 C<configure>. This may be useful when constructing an object to represent a
-network connection, before the C<connect()> has actually been performed yet.
+network connection, before the C<connect(2)> has actually been performed yet.
 
 =cut
 
@@ -171,12 +164,19 @@ sub configure
 
       if( defined $read_handle ) {
          if( !defined eval { $read_handle->fileno } ) {
-            croak 'Expected that read_handle can fileno()';
+            croak 'Expected that read_handle can ->fileno';
          }
 
-         if( !$self->{on_read_ready} and !$self->can( 'on_read_ready' ) ) {
+         unless( $self->can_event( 'on_read_ready' ) ) {
             croak 'Expected either a on_read_ready callback or an ->on_read_ready method';
          }
+
+         my @layers = PerlIO::get_layers( $read_handle );
+         if( grep m/^encoding\(/, @layers or grep m/^utf8$/, @layers ) {
+            # Only warn for now, because if it's UTF-8 by default but only
+            # passes ASCII then all will be well
+            carp "Constructing a ".ref($self)." with an encoding-enabled handle may not read correctly";
+         }
       }
 
       $self->{read_handle} = $read_handle;
@@ -192,10 +192,10 @@ sub configure
 
       if( defined $write_handle ) {
          if( !defined eval { $write_handle->fileno } ) {
-            croak 'Expected that write_handle can fileno()';
+            croak 'Expected that write_handle can ->fileno';
          }
 
-         if( !$self->{on_write_ready} and !$self->can( 'on_write_ready' ) ) {
+         unless( $self->can_event( 'on_write_ready' ) ) {
             # This used not to be fatal. Make it just a warning for now.
             carp 'A write handle was provided but neither a on_write_ready callback nor an ->on_write_ready method were. Perhaps you mean \'read_handle\' instead?';
          }
@@ -207,6 +207,10 @@ sub configure
       undef $self->{handle_closing};
    }
 
+   if( exists $params{want_readready} ) {
+      $self->want_readready( delete $params{want_readready} );
+   }
+
    if( exists $params{want_writeready} ) {
       $self->want_writeready( delete $params{want_writeready} );
    }
@@ -224,13 +228,11 @@ sub _watch_read
    my $self = shift;
    my ( $want ) = @_;
 
-   my $loop = $self->get_loop or return;
+   my $loop = $self->loop or return;
    my $fh = $self->read_handle or return;
 
    if( $want ) {
-      $self->{cb_r} ||= $self->{on_read_ready} ?
-         $self->_capture_weakself( $self->{on_read_ready} ) :
-         $self->_capture_weakself( 'on_read_ready' );
+      $self->{cb_r} ||= $self->make_event_cb( 'on_read_ready' );
 
       $loop->watch_io(
          handle => $fh,
@@ -250,13 +252,11 @@ sub _watch_write
    my $self = shift;
    my ( $want ) = @_;
 
-   my $loop = $self->get_loop or return;
+   my $loop = $self->loop or return;
    my $fh = $self->write_handle or return;
 
    if( $want ) {
-      $self->{cb_w} ||= $self->{on_write_ready} ?
-         $self->_capture_weakself( $self->{on_write_ready} ) :
-         $self->_capture_weakself( 'on_write_ready' );
+      $self->{cb_w} ||= $self->make_event_cb( 'on_write_ready' );
 
       $loop->watch_io(
          handle => $fh,
@@ -289,6 +289,22 @@ sub _remove_from_loop
    $self->_watch_write(0);
 }
 
+sub notifier_name
+{
+   my $self = shift;
+   if( length( my $name = $self->SUPER::notifier_name ) ) {
+      return $name;
+   }
+
+   my $r = $self->read_fileno;
+   my $w = $self->write_fileno;
+   return "rw=$r"     if defined $r and defined $w and $r == $w;
+   return "r=$r,w=$w" if defined $r and defined $w;
+   return "r=$r"      if defined $r;
+   return "w=$w"      if defined $w;
+   return "no";
+}
+
 =head1 METHODS
 
 =cut
@@ -342,30 +358,59 @@ sub close
    return if $self->{handle_closing};
    $self->{handle_closing} = 1;
 
-   if( $self->{on_closed} ) {
-      $self->{on_closed}->( $self );
-   }
-   elsif( $self->can( "on_closed" ) ) {
-      $self->on_closed;
-   }
-
-   if( my $parent = $self->{parent} ) {
-      $parent->remove_child( $self );
-   }
-   elsif( my $loop = $self->get_loop ) {
-      $loop->remove( $self );
-   }
+   $self->want_readready( 0 );
+   $self->want_writeready( 0 );
 
    my $read_handle = delete $self->{read_handle};
    $read_handle->close if defined $read_handle;
 
    my $write_handle = delete $self->{write_handle};
-   $write_handle->close if defined $write_handle and ( not defined $read_handle or $write_handle != $read_handle );
+   $write_handle->close if defined $write_handle;
 
-   # Clear the want* states, so if we get a new handle and are re-opened,
-   # we'll rearm the underlying loop
-   undef $self->{want_readready};
-   undef $self->{want_writeready};
+   $self->maybe_invoke_event( on_closed => );
+   $self->remove_from_parent;
+}
+
+=head2 $handle->close_read
+
+=head2 $handle->close_write
+
+Closes the underlying read or write handle, and deconfigures it from the
+object. Neither of these methods will invoke the C<on_closed> event, nor
+remove the object from the Loop if there is still one open handle in the
+object. Only when both handles are closed, will C<on_closed> be fired, and the
+object removed.
+
+=cut
+
+sub close_read
+{
+   my $self = shift;
+
+   $self->want_readready( 0 );
+
+   my $read_handle = delete $self->{read_handle};
+   $read_handle->close if defined $read_handle;
+
+   if( !$self->{write_handle} ) {
+      $self->maybe_invoke_event( on_closed => );
+      $self->remove_from_parent;
+   }
+}
+
+sub close_write
+{
+   my $self = shift;
+
+   $self->want_writeready( 0 );
+
+   my $write_handle = delete $self->{write_handle};
+   $write_handle->close if defined $write_handle;
+
+   if( !$self->{read_handle} ) {
+      $self->maybe_invoke_event( on_closed => );
+      $self->remove_from_parent;
+   }
 }
 
 =head2 $handle = $handle->read_handle
@@ -477,11 +522,6 @@ sub want_writeready
    }
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 SEE ALSO
 
 =over 4
@@ -495,3 +535,7 @@ L<IO::Handle> - Supply object methods for I/O handles
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
