@@ -9,13 +9,18 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.42';
+our $VERSION = '0.51';
 
 use IO::Async::Handle;
+use IO::Async::OS;
 
-use POSIX qw( EAGAIN );
+use Errno qw( EAGAIN EWOULDBLOCK );
 
-use Socket qw( sockaddr_family SOL_SOCKET SO_ACCEPTCONN SO_REUSEADDR SO_TYPE );
+use Socket qw(
+   sockaddr_family
+   SOL_SOCKET SO_ACCEPTCONN SO_REUSEADDR SO_TYPE
+   AF_INET6 IPPROTO_IPV6 IPV6_V6ONLY
+);
 
 use Carp;
 
@@ -28,7 +33,7 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
  use IO::Async::Listener;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  my $listener = IO::Async::Listener->new(
     on_stream => sub {
@@ -57,14 +62,14 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
     on_listen_error  => sub { print STDERR "Cannot listen\n"; },
  );
 
- $loop->loop_forever;
+ $loop->run;
 
 This object can also be used indirectly via an C<IO::Async::Loop>:
 
  use IO::Async::Stream;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  $loop->listen(
     service  => "echo",
@@ -78,7 +83,7 @@ This object can also be used indirectly via an C<IO::Async::Loop>:
     on_listen_error  => sub { print STDERR "Cannot listen\n"; },
  );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head1 DESCRIPTION
 
@@ -115,7 +120,8 @@ This is most useful for C<SOCK_DGRAM> or C<SOCK_RAW> sockets.
 =head2 on_accept_error $socket, $errno
 
 Optional. Invoked if the C<accept> syscall indicates an error (other than
-C<EAGAIN>). If not provided, failures of C<accept> will simply be ignored.
+C<EAGAIN> or C<EWOULDBLOCK>). If not provided, failures of C<accept> will
+simply be ignored.
 
 =cut
 
@@ -214,7 +220,10 @@ sub on_read_ready
          die "ARG! Missing on_accept,on_stream,on_socket!";
       }
    }
-   elsif( $! != EAGAIN ) {
+   elsif( $! == EAGAIN or $! == EWOULDBLOCK ) {
+      # ignore
+   }
+   else {
       $self->maybe_invoke_event( on_accept_error => $socket, $! );
    }
 }
@@ -296,7 +305,7 @@ is returned by the C<getaddrinfo> named resolver.
 
 Shortcut for passing a single address to listen on; it may be passed directly
 with this key, instead of in another array of its own. This should be in a
-format recognised by L<IO::Async::Loop>'s C<extract_addrinfo> method. See also
+format recognised by L<IO::Async::OS>'s C<extract_addrinfo> method. See also
 the C<EXAMPLES> section.
 
 =back
@@ -323,7 +332,7 @@ supplied.
 =item flags => INT
 
 Optional. Other arguments to pass along with C<host> and C<service> to the
-C<getaddrinfo()> call.
+C<getaddrinfo> call.
 
 =item socktype => STRING
 
@@ -362,8 +371,8 @@ Optional. A callback that is invoked when the listening socket is ready.
 A continuation this is invoked after all of the addresses have been tried, and
 none of them succeeded. It will be passed the most significant error that
 occurred, and the name of the operation it occurred in. Errors from the
-C<listen()> syscall are considered most significant, then C<bind()>, then
-C<sockopt()>, then finally C<socket()>.
+C<listen(2)> syscall are considered most significant, then C<bind(2)>, then
+C<sockopt(2)>, then finally C<socket(2)>.
 
 =item on_fail => CODE
 
@@ -381,7 +390,7 @@ the arguments that were passed to it, and the error generated. I.e.
 
 =item queuesize => INT
 
-Optional. The queue size to pass to the C<listen()> calls. If not supplied,
+Optional. The queue size to pass to the C<listen(2)> calls. If not supplied,
 then 3 will be given instead.
 
 =item reuseaddr => BOOL
@@ -389,6 +398,13 @@ then 3 will be given instead.
 Optional. If true or not supplied then the C<SO_REUSEADDR> socket option will
 be set. To prevent this, pass a false value such as 0.
 
+=item v6only => BOOL
+
+Optional. If defined, sets or clears the C<IPV6_V6ONLY> socket option on
+C<PF_INET6> sockets. This option disables the ability of C<PF_INET6> socket to
+accept connections from C<AF_INET> addresses. Not all operating systems allow
+this option to be disabled.
+
 =back
 
 As a convenience, it also supports a C<handle> argument, which is passed
@@ -401,7 +417,7 @@ sub listen
    my $self = shift;
    my ( %params ) = @_;
 
-   my $loop = $self->get_loop;
+   my $loop = $self->loop;
    defined $loop or croak "Cannot listen when not a member of a Loop"; # TODO: defer?
 
    if( exists $params{handle} ) {
@@ -430,36 +446,46 @@ sub listen
       my $reuseaddr = 1;
       $reuseaddr = 0 if defined $params{reuseaddr} and not $params{reuseaddr};
 
+      my $v6only = $params{v6only};
+
       my ( $listenerr, $binderr, $sockopterr, $socketerr );
 
       foreach my $addr ( @$addrlist ) {
-         my ( $family, $socktype, $proto, $address ) = $loop->extract_addrinfo( $addr );
+         my ( $family, $socktype, $proto, $address ) = IO::Async::OS->extract_addrinfo( $addr );
 
          my $sock;
 
-         unless( $sock = $loop->socket( $family, $socktype, $proto ) ) {
+         unless( $sock = IO::Async::OS->socket( $family, $socktype, $proto ) ) {
             $socketerr = $!;
-            $on_fail->( "socket", $family, $socktype, $proto, $! ) if $on_fail;
+            $on_fail->( socket => $family, $socktype, $proto, $! ) if $on_fail;
             next;
          }
 
          if( $reuseaddr ) {
             unless( $sock->sockopt( SO_REUSEADDR, 1 ) ) {
                $sockopterr = $!;
-               $on_fail->( "sockopt", $sock, SO_REUSEADDR, 1, $! ) if $on_fail;
+               $on_fail->( sockopt => $sock, SO_REUSEADDR, 1, $! ) if $on_fail;
+               next;
+            }
+         }
+
+         if( defined $v6only and $family == AF_INET6 ) {
+            unless( $sock->setsockopt( IPPROTO_IPV6, IPV6_V6ONLY, $v6only ) ) {
+               $sockopterr = $!;
+               $on_fail->( sockopt => $sock, IPV6_V6ONLY, $v6only, $! ) if $on_fail;
                next;
             }
          }
 
          unless( $sock->bind( $address ) ) {
             $binderr = $!;
-            $on_fail->( "bind", $sock, $address, $! ) if $on_fail;
+            $on_fail->( bind => $sock, $address, $! ) if $on_fail;
             next;
          }
 
          unless( $sock->listen( $queuesize ) ) {
             $listenerr = $!;
-            $on_fail->( "listen", $sock, $queuesize, $! ) if $on_fail;
+            $on_fail->( listen => $sock, $queuesize, $! ) if $on_fail;
             next;
          }
 
@@ -527,7 +553,7 @@ sockets.
  use IO::Socket::UNIX;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
 
  my $listener = IO::Async::Listener->new(
     on_stream => sub {
@@ -557,12 +583,12 @@ sockets.
     handle => $socket,
  );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head2 Passing Plain Socket Addresses
 
 The C<addr> or C<addrs> parameters should contain a definition of a plain
-socket address in a form that the L<IO::Async::Loop> C<extract_addrinfo>
+socket address in a form that the L<IO::Async::OS> C<extract_addrinfo>
 method can use.
 
 This example shows how to use the C<Socket> functions to construct one for
