@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.45';
+our $VERSION = '0.51';
 
 # When editing this value don't forget to update the docs below
 use constant NEED_API_VERSION => '0.33';
@@ -18,19 +18,12 @@ use constant _CAN_ON_HANGUP => 0;
 
 use Carp;
 
-use Socket qw( AF_INET AF_UNIX SOCK_STREAM SOCK_DGRAM SOCK_RAW );
-BEGIN {
-   # Not quite sure where we'll find AF_INET6
-   eval { Socket->import( 'AF_INET6' ); 1 } or
-      eval { require Socket6; Socket6->import( 'AF_INET6' ) }
-}
-use IO::Socket;
+use IO::Socket (); # empty import
 use Time::HiRes qw(); # empty import
 use POSIX qw( _exit WNOHANG );
 use Scalar::Util qw( refaddr );
 
-# Try to load IO::Socket::INET6 but don't worry if we don't have it
-# eval { require IO::Socket::INET6 };
+use IO::Async::OS;
 
 # Never sleep for more than 1 second if a signal proxy is registered, to avoid
 # a borderline race condition.
@@ -81,7 +74,7 @@ C<IO::Async::Loop> - core loop of the C<IO::Async> framework
     },
  ) );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head1 DESCRIPTION
 
@@ -90,7 +83,7 @@ C<IO::Async> framework. Its primary purpose is to store a set of
 L<IO::Async::Notifier> objects or subclasses of them. It handles all of the
 lower-level set manipulation actions, and leaves the actual IO readiness 
 testing/notification to the concrete class that implements it. It also
-provides other functionallity such as signal handling, child process managing,
+provides other functionality such as signal handling, child process managing,
 and timers.
 
 See also the two bundled Loop subclasses:
@@ -124,11 +117,11 @@ sub __new
       notifiers    => {}, # {nkey} = notifier
       iowatches    => {}, # {fd} = [ $on_read_ready, $on_write_ready, $on_hangup ]
       sigattaches  => {}, # {sig} => \@callbacks
-      sigproxy     => undef,
       childmanager => undef,
       childwatches => {}, # {pid} => $code
       timequeue    => undef,
       deferrals    => [],
+      os           => {}, # A generic scratchpad for IO::Async::OS to store whatever it wants
    }, $class;
 
    # It's possible this is a specific subclass constructor. We still want the
@@ -136,6 +129,14 @@ sub __new
    # one
    our $ONE_TRUE_LOOP ||= $self;
 
+   # Legacy support - temporary until all CPAN classes are updated; bump NEEDAPI version at that point
+   my $old_timer = $self->can( "enqueue_timer" ) != \&enqueue_timer;
+   if( $old_timer != ( $self->can( "cancel_timer" ) != \&cancel_timer ) ) {
+      die "$class should overload both ->enqueue_timer and ->cancel_timer, or neither";
+   }
+
+   $self->{old_timer} = $old_timer;
+
    return $self;
 }
 
@@ -403,38 +404,75 @@ sub loop_once
    croak "Expected that $self overrides ->loop_once";
 }
 
+=head2 @result = $loop->run
+
+=head2 $result = $loop->run
+
+Runs the actual IO event loop. This method blocks until the C<stop> method is
+called, and returns the result that was passed to C<stop>. In scalar context
+only the first result is returned; the others will be discarded if more than
+one value was provided. This method may be called recursively.
+
+This method is a recent addition and may not be supported by all the
+C<IO::Async::Loop> subclasses currently available on CPAN.
+
+=cut
+
+sub run
+{
+   my $self = shift;
+
+   local $self->{running} = 1;
+   local $self->{result} = [];
+
+   while( $self->{running} ) {
+      $self->loop_once( undef );
+   }
+
+   return wantarray ? @{ $self->{result} } : $self->{result}[0];
+}
+
+=head2 $loop->stop( @result )
+
+Stops the inner-most C<run> method currently in progress, causing it to return
+the given C<@result>.
+
+This method is a recent addition and may not be supported by all the
+C<IO::Async::Loop> subclasses currently available on CPAN.
+
+=cut
+
+sub stop
+{
+   my $self = shift;
+
+   @{ $self->{result} } = @_;
+   undef $self->{running};
+}
+
 =head2 $loop->loop_forever
 
-This method repeatedly calls the C<loop_once> method with no timeout (i.e.
-allowing the underlying mechanism to block indefinitely), until the
-C<loop_stop> method is called from an event callback.
+A synonym for C<run>, though this method does not return a result.
 
 =cut
 
 sub loop_forever
 {
    my $self = shift;
-
-   $self->{still_looping} = 1;
-
-   while( $self->{still_looping} ) {
-      $self->loop_once( undef );
-   }
+   $self->run;
+   return;
 }
 
 =head2 $loop->loop_stop
 
-This method cancels a running C<loop_forever>, and makes that method return.
-It would be called from an event callback triggered by an event that occured
-within the loop.
+A synonym for C<stop>, though this method does not pass any results.
 
 =cut
 
 sub loop_stop
 {
    my $self = shift;
-   
-   $self->{still_looping} = 0;
+   $self->stop;
 }
 
 ############
@@ -444,7 +482,7 @@ sub loop_stop
 =head1 FEATURES
 
 Most of the following methods are higher-level wrappers around base
-functionallity provided by the low-level API documented below. They may be
+functionality provided by the low-level API documented below. They may be
 used by C<IO::Async::Notifier> subclasses or called directly by the program.
 
 =cut
@@ -592,13 +630,7 @@ sub later
    return $self->watch_idle( when => 'later', code => $code );
 }
 
-=head2 $pid = $loop->detach_child( %params )
-
-This method creates a new child process to run a given code block. It is a
-legacy wrapper around the C<fork> method.
-
-=cut
-
+# undocumented, to be removed soon
 sub detach_child
 {
    my $self = shift;
@@ -606,18 +638,7 @@ sub detach_child
    $self->fork( @_ );
 }
 
-=head2 $code = $loop->detach_code( %params )
-
-This method creates a new detached code object. It is equivalent to calling
-the C<IO::Async::DetachedCode> constructor, passing in the given loop. See the
-documentation on this class for more information.
-
-Note that this behaviour is now deprecated, in favour of constructing a
-L<IO::Async::Function> object instead, and adding it to the loop. This object
-is more flexible and more powerful than the legacy C<DetachedCode>.
-
-=cut
-
+# undocumented, to be removed soon
 sub detach_code
 {
    my $self = shift;
@@ -1052,385 +1073,25 @@ to give different implementations on that OS.
 
 =cut
 
-# This one isn't documented because it's not really overridable. It's largely
-# here just for completeness
-sub socket
-{
-   my $self = shift;
-   my ( $family, $socktype, $proto ) = @_;
-
-   croak "Cannot create a new socket without a family" unless $family;
-
-   # SOCK_STREAM is the most likely
-   defined $socktype or $socktype = SOCK_STREAM;
-
-   defined $proto or $proto = 0;
-
-   my $sock = eval {
-      IO::Socket->new(
-         Domain => $family, 
-         Type   => $socktype,
-         Proto  => $proto,
-      );
-   };
-   return $sock if $sock;
-
-   # That failed. Most likely because the Domain was unrecognised. This 
-   # usually happens if getaddrinfo returns an AF_INET6 address but we don't
-   # have IO::Socket::INET6 loaded. In this case we'll return a generic one.
-   # It won't be in the specific subclass but that's the best we can do. And
-   # it will still work as a generic socket.
-   return IO::Socket->new->socket( $family, $socktype, $proto );
-}
-
-sub _getfamilybyname
-{
-   my ( $name ) = @_;
-
-   return undef unless defined $name;
-
-   return $name if $name =~ m/^\d+$/;
-
-   return AF_INET    if $name eq "inet";
-   return AF_INET6() if $name eq "inet6" and defined &AF_INET6;
-   return AF_UNIX    if $name eq "unix";
-
-   croak "Unrecognised socktype name '$name'";
-}
-
-sub _getsocktypebyname
-{
-   my ( $name ) = @_;
-
-   return undef unless defined $name;
-
-   return $name if $name =~ m/^\d+$/;
-
-   return SOCK_STREAM if $name eq "stream";
-   return SOCK_DGRAM  if $name eq "dgram";
-   return SOCK_RAW    if $name eq "raw";
-
-   croak "Unrecognised socktype name '$name'";
-}
-
 =head2 ( $S1, $S2 ) = $loop->socketpair( $family, $socktype, $proto )
 
-An abstraction of the C<socketpair(2)> syscall, where any argument may be
-missing (or given as C<undef>).
-
-If C<$family> is not provided, a suitable value will be provided by the OS
-(likely C<AF_UNIX> on POSIX-based platforms). If C<$socktype> is not provided,
-then C<SOCK_STREAM> will be used.
-
-Additionally, this method supports building connected C<SOCK_STREAM> or
-C<SOCK_DGRAM> pairs in the C<AF_INET> family even if the underlying platform's
-C<socketpair(2)> does not, by connecting two normal sockets together.
-
-C<$family> and C<$socktype> may also be given symbolically similar to the
-behaviour of C<extract_addrinfo>.
-
-=cut
-
-sub socketpair
-{
-   my $self = shift;
-   my ( $family, $socktype, $proto ) = @_;
-
-   # PF_UNSPEC and undef are both false
-   $family = _getfamilybyname( $family ) || AF_UNIX;
-
-   # SOCK_STREAM is the most likely
-   $socktype = _getsocktypebyname( $socktype ) || SOCK_STREAM;
-
-   $proto ||= 0;
-
-   my ( $S1, $S2 ) = IO::Socket->new->socketpair( $family, $socktype, $proto );
-   return ( $S1, $S2 ) if defined $S1;
-
-   return unless $family == AF_INET and ( $socktype == SOCK_STREAM or $socktype == SOCK_DGRAM );
-
-   # Now lets emulate an AF_INET socketpair call
-
-   my $Stmp = $self->socket( $family, $socktype ) or return;
-   $Stmp->bind( pack_sockaddr_in( 0, INADDR_LOOPBACK ) ) or return;
-
-   $S1 = $self->socket( $family, $socktype ) or return;
-
-   if( $socktype == SOCK_STREAM ) {
-      $Stmp->listen( 1 ) or return;
-      $S1->connect( getsockname $Stmp ) or return;
-      $S2 = $Stmp->accept or return;
-
-      # There's a bug in IO::Socket here, in that $S2 's ->socktype won't
-      # yet be set. We can apply a horribly hacky fix here
-      #   defined $S2->socktype and $S2->socktype == $socktype or
-      #     ${*$S2}{io_socket_type} = $socktype;
-      # But for now we'll skip the test for it instead
-   }
-   else {
-      $S2 = $Stmp;
-      $S1->connect( getsockname $S2 ) or return;
-      $S2->connect( getsockname $S1 ) or return;
-   }
-
-   return ( $S1, $S2 );
-}
-
 =head2 ( $rd, $wr ) = $loop->pipepair
 
-An abstraction of the C<pipe(2)> syscall, which returns the two new handles.
-
-=cut
-
-sub pipepair
-{
-   my $self = shift;
-
-   pipe( my ( $rd, $wr ) ) or return;
-   return ( $rd, $wr );
-}
-
 =head2 ( $rdA, $wrA, $rdB, $wrB ) = $loop->pipequad
 
-This method is intended for creating two pairs of filehandles that are linked
-together, suitable for passing as the STDIN/STDOUT pair to a child process.
-After this function returns, C<$rdA> and C<$wrA> will be a linked pair, as
-will C<$rdB> and C<$wrB>.
-
-On platforms that support C<socketpair(2)>, this implementation will be
-preferred, in which case C<$rdA> and C<$wrB> will actually be the same
-filehandle, as will C<$rdB> and C<$wrA>. This saves a file descriptor in the
-parent process.
-
-When creating a C<IO::Async::Stream> or subclass of it, the C<read_handle>
-and C<write_handle> parameters should always be used.
-
- my ( $childRd, $myWr, $myRd, $childWr ) = $loop->pipequad;
-
- $loop->open_child(
-    stdin  => $childRd,
-    stdout => $childWr,
-    ...
- );
-
- my $str = IO::Async::Stream->new(
-    read_handle  => $myRd,
-    write_handle => $myWr,
-    ...
- );
- $loop->add( $str );
-
-=cut
-
-sub pipequad
-{
-   my $self = shift;
-
-   # Prefer socketpair
-   if( my ( $S1, $S2 ) = $self->socketpair ) {
-      return ( $S1, $S2, $S2, $S1 );
-   }
-
-   # Can't do that, fallback on pipes
-   my ( $rdA, $wrA ) = $self->pipepair or return;
-   my ( $rdB, $wrB ) = $self->pipepair or return;
-
-   return ( $rdA, $wrA, $rdB, $wrB );
-}
-
 =head2 $signum = $loop->signame2num( $signame )
 
-This utility method converts a signal name (such as "TERM") into its system-
-specific signal number. This may be useful to pass to C<POSIX::SigSet> or use
-in other places which use numbers instead of symbolic names.
+Legacy wrappers around L<IO::Async::OS> functions.
 
 =cut
 
-my %sig_num;
-sub _init_signum
-{
-   my $self = shift;
-   # Copypasta from Config.pm's documentation
+sub socketpair  { shift; IO::Async::OS->socketpair( @_ ) }
+sub pipepair    { shift; IO::Async::OS->pipepair( @_ ) }
+sub pipequad    { shift; IO::Async::OS->pipequad( @_ ) }
+sub signame2num { shift; IO::Async::OS->signame2num( @_ ) }
 
-   our %Config;
-   require Config;
-   Config->import;
-
-   unless($Config{sig_name} && $Config{sig_num}) {
-      die "No signals found";
-   }
-   else {
-      my @names = split ' ', $Config{sig_name};
-      @sig_num{@names} = split ' ', $Config{sig_num};
-   }
-}
-
-sub signame2num
-{
-   my $self = shift;
-   my ( $signame ) = @_;
-
-   %sig_num or $self->_init_signum;
-
-   return $sig_num{$signame};
-}
-
-=head2 ( $family, $socktype, $protocol, $addr ) = $loop->extract_addrinfo( $ai )
-
-Given an ARRAY or HASH reference value containing an addrinfo, returns a
-family, socktype and protocol argument suitable for a C<socket> call and an
-address suitable for C<connect> or C<bind>.
-
-If given an ARRAY it should be in the following form:
-
- [ $family, $socktype, $protocol, $addr ]
-
-If given a HASH it should contain the following keys:
-
- family socktype protocol addr
-
-Each field in the result will be initialised to 0 (or empty string for the
-address) if not defined in the C<$ai> value.
-
-The family type may also be given as a symbolic string; C<inet> or possibly
-C<inet6> if the host system supports it, or C<unix>; this will be converted to
-the appropriate C<AF_*> constant.
-
-The socktype may also be given as a symbolic string; C<stream>, C<dgram> or
-C<raw>; this will be converted to the appropriate C<SOCK_*> constant.
-
-Note that the C<addr> field, if provided, must be a packed socket address,
-such as returned by C<pack_sockaddr_in> or C<pack_sockaddr_un>.
-
-If the HASH form is used, rather than passing a packed socket address in the
-C<addr> field, certain other hash keys may be used instead for convenience on
-certain named families.
-
-=over 4
-
-=cut
-
-use constant ADDRINFO_FAMILY   => 0;
-use constant ADDRINFO_SOCKTYPE => 1;
-use constant ADDRINFO_PROTOCOL => 2;
-use constant ADDRINFO_ADDR     => 3;
-
-sub extract_addrinfo
-{
-   my $self = shift;
-   my ( $ai, $argname ) = @_;
-
-   $argname ||= "addr";
-
-   my @ai;
-
-   if( ref $ai eq "ARRAY" ) {
-      @ai = @$ai;
-   }
-   elsif( ref $ai eq "HASH" ) {
-      @ai = @{$ai}{qw( family socktype protocol addr )};
-   }
-   else {
-      croak "Expected '$argname' to be an ARRAY or HASH reference";
-   }
-
-   if( defined $ai[ADDRINFO_FAMILY] and !defined $ai[ADDRINFO_ADDR] and ref $ai eq "HASH" ) {
-      my $family = $ai[ADDRINFO_FAMILY];
-      my $method = "_extract_addrinfo_$family";
-      my $code = $self->can( $method ) or croak "Cannot determine addr for extract_addrinfo on family='$family'";
-
-      $ai[ADDRINFO_ADDR] = $code->( $self, $ai );
-   }
-
-   $ai[ADDRINFO_FAMILY]   = _getfamilybyname( $ai[ADDRINFO_FAMILY] );
-   $ai[ADDRINFO_SOCKTYPE] = _getsocktypebyname( $ai[ADDRINFO_SOCKTYPE] );
-
-   # Make sure all fields are defined
-   $ai[$_] ||= 0 for ADDRINFO_FAMILY, ADDRINFO_SOCKTYPE, ADDRINFO_PROTOCOL;
-   $ai[ADDRINFO_ADDR]  = "" if !defined $ai[ADDRINFO_ADDR];
-
-   return @ai;
-}
-
-=item family => 'inet'
-
-Will pack an IP address and port number from keys called C<ip> and C<port>.
-
-=cut
-
-sub _extract_addrinfo_inet
-{
-   my $self = shift;
-   my ( $ai ) = @_;
-
-   defined( my $port = $ai->{port} ) or croak "Expected 'port' for extract_addrinfo on family='inet'";
-   defined( my $ip   = $ai->{ip}   ) or croak "Expected 'ip' for extract_addrinfo on family='inet'";
-
-   return Socket::pack_sockaddr_in( $port, Socket::inet_aton( $ip ) );
-}
-
-=item family => 'inet6'
-
-Will pack an IP address and port number from keys called C<ip> and C<port>.
-Optionally will also include values from C<scopeid> and C<flowinfo> keys if
-provided.
-
-This will only work if a C<pack_sockaddr_in6> function can be found, either
-in C<Socket> or C<Socket6>.
-
-=cut
-
-sub _extract_addrinfo_inet6
-{
-   my $self = shift;
-   my ( $ai ) = @_;
-
-   defined( my $port = $ai->{port} ) or croak "Expected 'port' for extract_addrinfo on family='inet6'";
-   defined( my $ip   = $ai->{ip}   ) or croak "Expected 'ip' for extract_addrinfo on family='inet6'";
-
-   my $scopeid  = $ai->{scopeid}  || 0;
-   my $flowinfo = $ai->{flowinfo} || 0;
-
-   # We're not quite sure where pack_sockaddr_in6 might come from. Perl's
-   # Socket module added it in 5.13.8 but before then Socket6 is our best bet.
-   # Socket6 isn't core though.
-   if( defined &Socket::pack_sockaddr_in6 ) {
-      return Socket::pack_sockaddr_in6( $port, Socket::inet_pton( Socket::AF_INET6(), $ip ), $scopeid, $flowinfo );
-   }
-   elsif( defined &Socket6::pack_sockaddr_in6_all ) {
-      return Socket6::pack_sockaddr_in6_all( $port, $flowinfo, Socket6::inet_pton( Socket6::AF_INET6(), $ip ), $scopeid );
-   }
-   else {
-      croak "Cannot pack_sockaddr_in6";
-   }
-}
-
-=item family => 'unix'
-
-Will pack a UNIX socket path from a key called C<path>.
-
-=cut
-
-sub _extract_addrinfo_unix
-{
-   my $self = shift;
-   my ( $ai ) = @_;
-
-   defined( my $path = $ai->{path} ) or croak "Expected 'path' for extract_addrinfo on family='unix'";
-
-   return Socket::pack_sockaddr_un( $path );
-}
-
-=pod
-
-=back
-
-This method used to be called C<unpack_addrinfo>. A backward compatibility
-wrapper is provided temporarily, but will be removed in a later version.
-
-=cut
-
-sub unpack_addrinfo { goto &extract_addrinfo }
+sub extract_addrinfo { shift; IO::Async::OS->extract_addrinfo( @_ ) }
+sub unpack_addrinfo  { goto &extract_addrinfo }
 
 =head2 $time = $loop->time
 
@@ -1521,7 +1182,7 @@ sub fork
 
 As C<IO::Async::Loop> is an abstract base class, specific subclasses of it are
 required to implement certain methods that form the base level of
-functionallity. They are not recommended for applications to use; see instead
+functionality. They are not recommended for applications to use; see instead
 the various event objects or higher level methods listed above.
 
 These methods should be considered as part of the interface contract required
@@ -1537,11 +1198,11 @@ required API. This method should return the API version that the loop
 implementation supports. The magic constructor will use that class, provided
 it declares a version at least as new as the version documented here.
 
-The current API version is C<0.33>.
+The current API version is C<0.49>.
 
 This method may be implemented using C<constant>; e.g
 
- use constant API_VERSION => '0.33';
+ use constant API_VERSION => '0.49';
 
 =cut
 
@@ -1699,8 +1360,7 @@ sub watch_signal
    my $self = shift;
    my ( $signal, $code ) = @_;
 
-   my $sigproxy = $self->{sigproxy} ||= $self->__new_feature( "IO::Async::Internals::SignalProxy" );
-   $sigproxy->watch( $signal, $code );
+   IO::Async::OS->loop_watch_signal( $self, $signal, $code );
 }
 
 =head2 $loop->unwatch_signal( $signal )
@@ -1722,17 +1382,117 @@ sub unwatch_signal
    my $self = shift;
    my ( $signal ) = @_;
 
-   my $sigproxy = $self->{sigproxy} ||= $self->__new_feature( "IO::Async::Internals::SignalProxy" );
-   $sigproxy->unwatch( $signal );
+   IO::Async::OS->loop_unwatch_signal( $self, $signal );
+}
 
-   if( !$sigproxy->signals ) {
-      $self->remove( $sigproxy );
-      undef $sigproxy;
-      undef $self->{sigproxy};
+=head2 $id = $loop->watch_time( %args )
+
+This method installs a callback which will be called at the specified time.
+The time may either be specified as an absolute value (the C<at> key), or
+as a delay from the time it is installed (the C<after> key).
+
+The returned C<$id> value can be used to identify the timer in case it needs
+to be cancelled by the C<unwatch_time> method. Note that this value may be
+an object reference, so if it is stored, it should be released after it has
+been fired or cancelled, so the object itself can be freed.
+
+The C<%params> hash takes the following keys:
+
+=over 8
+
+=item at => NUM
+
+The absolute system timestamp to run the event.
+
+=item after => NUM
+
+The delay after now at which to run the event, if C<at> is not supplied. A
+zero or negative delayed timer should be executed as soon as possible; the
+next time the C<loop_once> method is invoked.
+
+=item now => NUM
+
+The time to consider as now if calculating an absolute time based on C<after>;
+defaults to C<time()> if not specified.
+
+=item code => CODE
+
+CODE reference to the continuation to run at the allotted time.
+
+=back
+
+Either one of C<at> or C<after> is required.
+
+For more powerful timer functionality as a C<IO::Async::Notifier> (so it can
+be used as a child within another Notifier), see instead the
+L<IO::Async::Timer> object and its subclasses.
+
+These C<*_time> methods are optional; a subclass may implement neither or both
+of them. If it implements neither, then the base class will manage a queue of
+timer events. This queue should be handled by the C<loop_once> method
+implemented by the subclass, using the C<_adjust_timeout> and
+C<_manage_queues> methods.
+
+This is the newer version of the API, replacing C<enqueue_timer>. It is
+unspecified how this method pair interacts with the older
+C<enqueue/requeue/cancel_timer> triplet.
+
+=cut
+
+sub watch_time
+{
+   my $self = shift;
+   my %args = @_;
+
+   # Renamed args
+   if( exists $args{after} ) {
+      $args{delay} = delete $args{after};
+   }
+   elsif( exists $args{at} ) {
+      $args{time}  = delete $args{at};
+   }
+   else {
+      croak "Expected one of 'at' or 'after'";
+   }
+
+   if( $self->{old_timer} ) {
+      $self->enqueue_timer( %args );
+   }
+   else {
+      my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::Internals::TimeQueue" );
+
+      my $time = $self->_build_time( %args );
+      my $code = $args{code};
+
+      $timequeue->enqueue( time => $time, code => $code );
+   }
+}
+
+=head2 $loop->unwatch_time( $id )
+
+Removes a timer callback previously created by C<watch_time>.
+
+This is the newer version of the API, replacing C<cancel_timer>. It is
+unspecified how this method pair interacts with the older
+C<enqueue/requeue/cancel_timer> triplet.
+
+=cut
+
+sub unwatch_time
+{
+   my $self = shift;
+   my ( $id ) = @_;
+
+   if( $self->{old_timer} ) {
+      $self->cancel_timer( $id );
+   }
+   else {
+      my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::Internals::TimeQueue" );
+
+      $timequeue->cancel( $id );
    }
 }
 
-# For subclasses to call
 sub _build_time
 {
    my $self = shift;
@@ -1756,50 +1516,11 @@ sub _build_time
 
 =head2 $id = $loop->enqueue_timer( %params )
 
-This method installs a callback which will be called at the specified time.
-The time may either be specified as an absolute value (the C<time> key), or
-as a delay from the time it is installed (the C<delay> key).
-
-The returned C<$id> value can be used to identify the timer in case it needs
-to be cancelled by the C<cancel_timer> method. Note that this value may be
-an object reference, so if it is stored, it should be released after it has
-been fired or cancelled, so the object itself can be freed.
-
-The C<%params> hash takes the following keys:
-
-=over 8
-
-=item time => NUM
-
-The absolute system timestamp to run the event.
-
-=item delay => NUM
-
-The delay after now at which to run the event, if C<time> is not supplied. A
-zero or negative delayed timer should be executed as soon as possible; the
-next time the C<loop_once> method is invoked.
-
-=item now => NUM
-
-The time to consider as now if calculating an absolute time based on C<delay>;
-defaults to C<time> if not specified.
-
-=item code => CODE
-
-CODE reference to the continuation to run at the allotted time.
-
-=back
-
-Either one of C<time> or C<delay> is required.
-
-For more powerful timer functionallity as a C<IO::Async::Notifier> (so it can
-be used as a child within another Notifier), see instead the
-L<IO::Async::Timer> object and its subclasses.
-
-These C<*_timer> methods are optional; a subclass may implement none or all of
-them. If it implements none, then the base class will manage a queue of timer
-events. This queue should be handled by the C<loop_once> method implemented by
-the subclass, using the C<_adjust_timeout> and C<_manage_queues> methods.
+An older version of C<watch_time>. This method should not be used in new code
+but is retained for legacy purposes. For simple watch/unwatch behaviour use
+instead the new C<watch_time> method; though note it has differently-named
+arguments. For requeueable timers, consider using an
+L<IO::Async::Timer::Countdown> or L<IO::Async::Timer::Absolute> instead.
 
 =cut
 
@@ -1808,16 +1529,18 @@ sub enqueue_timer
    my $self = shift;
    my ( %params ) = @_;
 
-   my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::Internals::TimeQueue" );
+   # Renamed args
+   $params{after} = delete $params{delay} if exists $params{delay};
+   $params{at}    = delete $params{time}  if exists $params{time};
 
-   $params{time} = $self->_build_time( %params );
-
-   $timequeue->enqueue( %params );
+   my $code = $params{code};
+   return [ $self->watch_time( %params ), $code ];
 }
 
 =head2 $loop->cancel_timer( $id )
 
-Cancels a previously-enqueued timer event by removing it from the queue.
+An older version of C<unwatch_time>. This method should not be used in new
+code but is retained for legacy purposes.
 
 =cut
 
@@ -1825,10 +1548,7 @@ sub cancel_timer
 {
    my $self = shift;
    my ( $id ) = @_;
-
-   my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::Internals::TimeQueue" );
-
-   $timequeue->cancel( $id );
+   $self->unwatch_time( $id->[0] );
 }
 
 =head2 $newid = $loop->requeue_timer( $id, %params )
@@ -1843,6 +1563,10 @@ The requeue operation may be implemented as a cancel + enqueue, which may
 mean the ID changes. Be sure to store the returned C<$newid> value if it is
 required.
 
+This method should not be used in new code but is retained for legacy
+purposes. For requeueable, consider using an L<IO::Async::Timer::Countdown> or
+L<IO::Async::Timer::Absolute> instead.
+
 =cut
 
 sub requeue_timer
@@ -1850,11 +1574,8 @@ sub requeue_timer
    my $self = shift;
    my ( $id, %params ) = @_;
 
-   my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::Internals::TimeQueue" );
-
-   $params{time} = $self->_build_time( %params );
-
-   $timequeue->requeue( $id, %params );
+   $self->unwatch_time( $id->[0] );
+   return $self->enqueue_timer( %params, code => $id->[1] );
 }
 
 =head2 $id = $loop->watch_idle( %params )
@@ -2045,7 +1766,7 @@ sub unwatch_child
 =head1 METHODS FOR SUBCLASSES
 
 The following methods are provided to access internal features which are
-required by specific subclasses to implement the loop functionallity. The use
+required by specific subclasses to implement the loop functionality. The use
 cases of each will be documented in the above section.
 
 =cut
@@ -2119,7 +1840,7 @@ sub _manage_queues
 
 An Extension is a Perl module that provides extra methods in the
 C<IO::Async::Loop> or other packages. They are intended to provide extra
-functionallity that easily integrates with the rest of the code.
+functionality that easily integrates with the rest of the code.
 
 Certain base methods take an C<extensions> parameter; an ARRAY reference
 containing a list of extension names. If such a list is passed to a method, it
@@ -2145,7 +1866,7 @@ will become
 This is provided so that extension modules, such as L<IO::Async::SSL> can
 easily be invoked indirectly, by passing extra arguments to C<connect> methods
 or similar, without needing every module to be aware of the C<SSL> extension.
-This functionallity is generic and not limited to C<SSL>; other extensions may
+This functionality is generic and not limited to C<SSL>; other extensions may
 also use it.
 
 The following methods take an C<extensions> parameter:
