@@ -1,15 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop::Poll;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.45';
-use constant API_VERSION => '0.33';
+our $VERSION = '0.51';
+use constant API_VERSION => '0.49';
 
 use base qw( IO::Async::Loop );
 
@@ -17,7 +17,7 @@ use Carp;
 
 use IO::Poll qw( POLLIN POLLOUT POLLHUP POLLERR );
 
-use POSIX qw( EINTR );
+use POSIX qw( EINTR S_ISREG );
 
 # IO::Poll version 0.05 contain a bug whereby the ->remove method doesn't
 # properly clean up all the references to the handles. If the version we're
@@ -30,6 +30,10 @@ use constant _CAN_ON_HANGUP =>
    ( $^O eq "linux" ) ||
    ( $^O eq "freebsd" and do { no warnings 'numeric'; (POSIX::uname)[2] >= 8.0 } );
 
+# poll() on most platforms claims that ISREG files are always read- and
+# write-ready, but not on MSWin32. We need to fake this
+use constant FAKE_ISREG_READY => IO::Async::OS->HAVE_FAKE_ISREG_READY;
+
 =head1 NAME
 
 C<IO::Async::Loop::Poll> - use C<IO::Async> with C<poll(2)>
@@ -115,7 +119,7 @@ This method checks the returned event list from a C<IO::Poll::poll> call,
 and calls any of the notification methods or callbacks that are appropriate.
 It returns the total number of callbacks that were invoked; that is, the
 total number of C<on_read_ready> and C<on_write_ready> callbacks for 
-C<watch_io>, and C<enqueue_timer> event callbacks.
+C<watch_io>, and C<watch_time> event callbacks.
 
 =over 8
 
@@ -140,6 +144,9 @@ sub post_poll
       my $watch = $iowatches->{$fd} or next;
 
       my $events = $poll->events( $watch->[0] );
+      if( FAKE_ISREG_READY and $self->{fake_isreg}{$fd} ) {
+         $events |= $self->{fake_isreg}{$fd} & ( POLLIN|POLLOUT );
+      }
 
       # We have to test separately because kernel doesn't report POLLIN when
       # a pipe gets closed.
@@ -151,7 +158,7 @@ sub post_poll
          $count++, $watch->[2]->() if defined $watch->[2];
       }
 
-      if( $events & POLLHUP ) {
+      if( $events & (POLLHUP|POLLERR) ) {
          $count++, $watch->[3]->() if defined $watch->[3];
       }
    }
@@ -180,6 +187,15 @@ sub loop_once
 
    $self->_adjust_timeout( \$timeout );
 
+   $timeout = 0 if FAKE_ISREG_READY and keys %{ $self->{fake_isreg} };
+
+   # Round up to nearest millisecond
+   if( $timeout ) {
+      my $mils = $timeout * 1000;
+      my $fraction = $mils - int $mils;
+      $timeout += ( 1 - $fraction ) / 1000 if $fraction;
+   }
+
    my $poll = $self->{poll};
 
    my $pollret;
@@ -232,6 +248,10 @@ sub watch_io
    $params{on_write_ready} and $mask |= POLLOUT;
    $params{on_hangup}      and $mask |= POLLHUP;
 
+   if( FAKE_ISREG_READY and S_ISREG(stat $handle) ) {
+      $self->{fake_isreg}{$handle->fileno} = [ $handle, $mask ];
+   }
+
    $poll->mask( $handle, $mask ) if $mask != $curmask;
 }
 
@@ -254,6 +274,15 @@ sub unwatch_io
    $params{on_write_ready} and $mask &= ~POLLOUT;
    $params{on_hangup}      and $mask &= ~POLLHUP;
 
+   if( FAKE_ISREG_READY and S_ISREG(stat $handle) ) {
+      if( $mask ) {
+         $self->{fake_isreg}{$handle->fileno} = [ $handle, $mask ];
+      }
+      else {
+         delete $self->{fake_isreg}{$handle->fileno};
+      }
+   }
+
    $poll->mask( $handle, $mask ) if $mask != $curmask;
 }
 
