@@ -1,43 +1,47 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop::Select;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
-use constant API_VERSION => '0.24';
+our $VERSION = '0.51';
+use constant API_VERSION => '0.49';
 
 use base qw( IO::Async::Loop );
 
+use IO::Async::OS;
+
 use Carp;
 
+use POSIX qw( S_ISREG );
+
+# select() on most platforms claims that ISREG files are always read- and
+# write-ready, but not on MSWin32. We need to fake this
+use constant FAKE_ISREG_READY => IO::Async::OS->HAVE_FAKE_ISREG_READY;
+# select() on most platforms indicates write-ready when connect() fails, but
+# not on MSWin32. Have to pull from evec in that case
+use constant SELECT_CONNECT_EVEC => IO::Async::OS->HAVE_SELECT_CONNECT_EVEC;
+
 =head1 NAME
 
 C<IO::Async::Loop::Select> - use C<IO::Async> with C<select(2)>
 
 =head1 SYNOPSIS
 
+Normally an instance of this class would not be directly constructed by a
+program. It may however, be useful for runinng L<IO::Async> with an existing
+program already using a C<select> call.
+
  use IO::Async::Loop::Select;
 
- my $loop = IO::Async::Loop::Select->new();
+ my $loop = IO::Async::Loop::Select->new;
 
  $loop->add( ... );
 
- $loop->loop_forever();
-
-Or
-
- while(1) {
-    $loop->loop_once();
-    ...
- }
-
-Or
-
  while(1) {
     my ( $rvec, $wvec, $evec ) = ('') x 3;
     my $timeout;
@@ -51,16 +55,14 @@ Or
 
 =head1 DESCRIPTION
 
-This subclass of C<IO::Async::Loop> uses the C<select()> syscall to perform
+This subclass of C<IO::Async::Loop> uses the C<select(2)> syscall to perform
 read-ready and write-ready tests.
 
-To integrate with an existing C<select()>-based event loop, a pair of methods
-C<pre_select()> and C<post_select()> can be called immediately before and
-after a C<select()> call. The relevant bit in the read-ready bitvector is
-always set by the C<pre_select()> method, but the corresponding bit in
-write-ready vector is set depending on the state of the C<'want_writeready'>
-property. The C<post_select()> method will invoke the C<on_read_ready()> or
-C<on_write_ready()> methods or callbacks as appropriate.
+To integrate with an existing C<select>-based event loop, a pair of methods
+C<pre_select> and C<post_select> can be called immediately before and
+after a C<select> call. The relevant bits in the read-ready, write-ready and
+exceptional-state bitvectors are set by the C<pre_select> method, and tested
+by the C<post_select> method to pick which event callbacks to invoke.
 
 =cut
 
@@ -68,7 +70,7 @@ C<on_write_ready()> methods or callbacks as appropriate.
 
 =cut
 
-=head2 $loop = IO::Async::Loop::Select->new()
+=head2 $loop = IO::Async::Loop::Select->new
 
 This function returns a new instance of a C<IO::Async::Loop::Select> object.
 It takes no special arguments.
@@ -83,6 +85,9 @@ sub new
 
    $self->{rvec} = '';
    $self->{wvec} = '';
+   $self->{evec} = '';
+
+   $self->{avec} = ''; # Bitvector of handles always to claim are ready
 
    return $self;
 }
@@ -93,11 +98,10 @@ sub new
 
 =head2 $loop->pre_select( \$readvec, \$writevec, \$exceptvec, \$timeout )
 
-This method prepares the bitvectors for a C<select()> call, setting the bits
-that notifiers registered by this loop are interested in. It will always set
-the appropriate bits in the read vector, but will only set them in the write
-vector if the notifier's C<want_writeready()> property is true. Neither the
-exception vector nor the timeout are affected.
+This method prepares the bitvectors for a C<select> call, setting the bits
+that the Loop is interested in. It will also adjust the C<$timeout> value if
+appropriate, reducing it if the next event timeout the Loop requires is sooner
+than the current value.
 
 =over 8
 
@@ -123,18 +127,28 @@ sub pre_select
    my ( $readref, $writeref, $exceptref, $timeref ) = @_;
 
    # BITWISE operations
-   $$readref  |= $self->{rvec};
-   $$writeref |= $self->{wvec};
+   $$readref   |= $self->{rvec};
+   $$writeref  |= $self->{wvec};
+   $$exceptref |= $self->{evec};
 
    $self->_adjust_timeout( $timeref );
 
+   $$timeref = 0 if FAKE_ISREG_READY and length $self->{avec};
+
+   # Round up to nearest millisecond
+   if( $$timeref ) {
+      my $mils = $$timeref * 1000;
+      my $fraction = $mils - int $mils;
+      $$timeref += ( 1 - $fraction ) / 1000 if $fraction;
+   }
+
    return;
 }
 
 =head2 $loop->post_select( $readvec, $writevec, $exceptvec )
 
-This method checks the returned bitvectors from a C<select()> call, and calls
-any of the notification methods or callbacks that are appropriate.
+This method checks the returned bitvectors from a C<select> call, and calls
+any of the callbacks that are appropriate.
 
 =over 8
 
@@ -164,11 +178,14 @@ sub post_select
 
       my $fileno = $watch->[0]->fileno;
 
-      if( vec( $readvec, $fileno, 1 ) ) {
+      if( vec( $readvec, $fileno, 1 ) or 
+          FAKE_ISREG_READY and vec( $self->{avec}, $fileno, 1 ) and vec( $self->{rvec}, $fileno, 1 ) ) {
          $count++, $watch->[1]->() if defined $watch->[1];
       }
 
-      if( vec( $writevec, $fileno, 1 ) ) {
+      if( vec( $writevec, $fileno, 1 ) or
+          SELECT_CONNECT_EVEC and vec( $exceptvec, $fileno, 1 ) or
+          FAKE_ISREG_READY and vec( $self->{avec}, $fileno, 1 ) and vec( $self->{wvec}, $fileno, 1 ) ) {
          $count++, $watch->[2]->() if defined $watch->[2];
       }
    }
@@ -181,15 +198,14 @@ sub post_select
 
 =head2 $count = $loop->loop_once( $timeout )
 
-This method calls the C<pre_select()> method to prepare the bitvectors for a
-C<select()> syscall, performs it, then calls C<post_select()> to process the
+This method calls the C<pre_select> method to prepare the bitvectors for a
+C<select> syscall, performs it, then calls C<post_select> to process the
 result. It returns the total number of callbacks invoked by the
-C<post_select()> method, or C<undef> if the underlying C<select()> syscall
+C<post_select> method, or C<undef> if the underlying C<select(2)> syscall
 returned an error.
 
 =cut
 
-# override
 sub loop_once
 {
    my $self = shift;
@@ -220,6 +236,12 @@ sub watch_io
 
    vec( $self->{rvec}, $fileno, 1 ) = 1 if $params{on_read_ready};
    vec( $self->{wvec}, $fileno, 1 ) = 1 if $params{on_write_ready};
+
+   # MSWin32 does not indicate writeready for connect() errors, HUPs, etc
+   # but it does indicate exceptional
+   vec( $self->{evec}, $fileno, 1 ) = 1 if SELECT_CONNECT_EVEC and $params{on_write_ready};
+
+   vec( $self->{avec}, $fileno, 1 ) = 1 if FAKE_ISREG_READY and S_ISREG(stat $params{handle});
 }
 
 sub unwatch_io
@@ -234,16 +256,15 @@ sub unwatch_io
    vec( $self->{rvec}, $fileno, 1 ) = 0 if $params{on_read_ready};
    vec( $self->{wvec}, $fileno, 1 ) = 0 if $params{on_write_ready};
 
-   # vec() will grow a bit vector as needed, but never shrink it. We'll trim
+   vec( $self->{evec}, $fileno, 1 ) = 0 if SELECT_CONNECT_EVEC and $params{on_write_ready};
+
+   vec( $self->{avec}, $fileno, 1 ) = 0 if FAKE_ISREG_READY and S_ISREG(stat $params{handle});
+
+   # vec will grow a bit vector as needed, but never shrink it. We'll trim
    # trailing null bytes
-   $_ =~s/\0+\z// for $self->{rvec}, $self->{wvec};
+   $_ =~s/\0+\z// for $self->{rvec}, $self->{wvec}, $self->{evec}, $self->{avec};
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 SEE ALSO
 
 =over 4
@@ -257,3 +278,7 @@ L<IO::Select> - OO interface to select system call
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
