@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::LoopTests;
 
@@ -19,12 +19,15 @@ use Test::Refcount;
 
 use IO::Async::Test qw();
 
+use IO::Async::OS;
+
 use IO::File;
 use Fcntl qw( SEEK_SET );
 use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
+use Socket qw( sockaddr_family AF_UNIX );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.45';
+our $VERSION = '0.51';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
@@ -84,7 +87,7 @@ sub run_tests
    my ( $testclass, @tests ) = @_;
 
    my $count = 0;
-   $count += __PACKAGE__->can( "count_tests_$_" )->() + 3 for @tests;
+   $count += __PACKAGE__->can( "count_tests_$_" )->() + 4 for @tests;
 
    plan tests => $count;
 
@@ -98,6 +101,8 @@ sub run_tests
    foreach my $test ( @tests ) {
       $loop = $testclass->new;
 
+      isa_ok( $loop, $testclass, '$loop' );
+
       is( IO::Async::Loop->new, $loop, 'magic constructor yields $loop' );
 
       # Kill the reference in $ONE_TRUE_LOOP so as not to upset the refcounts
@@ -152,11 +157,11 @@ Tests the Loop's ability to watch filehandles for IO readiness
 
 =cut
 
-use constant count_tests_io => 17;
+use constant count_tests_io => 16;
 sub run_tests_io
 {
    {
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot create socket pair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot create socket pair - $!";
       $_->blocking( 0 ) for $S1, $S2;
 
       my $readready  = 0;
@@ -222,7 +227,7 @@ sub run_tests_io
 
    # HUP of pipe - can be different to sockets on some architectures
    {
-      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      my ( $Prd, $Pwr ) = IO::Async::OS->pipepair or die "Cannot pipepair - $!";
       $_->blocking( 0 ) for $Prd, $Pwr;
 
       my $readready = 0;
@@ -249,42 +254,31 @@ sub run_tests_io
    }
 
    SKIP: {
-      $loop->_CAN_ON_HANGUP or skip "Loop cannot watch_io for on_hangup", 3;
+      $loop->_CAN_ON_HANGUP or skip "Loop cannot watch_io for on_hangup", 2;
 
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
-      $_->blocking( 0 ) for $S1, $S2;
+      SKIP: {
+         my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot socketpair - $!";
+         $_->blocking( 0 ) for $S1, $S2;
+
+         sockaddr_family( $S1->sockname ) == AF_UNIX or skip "Cannot reliably detect hangup condition on non AF_UNIX sockets", 1;
+
+         my $hangup = 0;
+         $loop->watch_io(
+            handle => $S1,
+            on_hangup => sub { $hangup = 1 },
+         );
+
+         $S2->close;
+
+         $loop->loop_once( 0.1 );
+
+         is( $hangup, 1, '$hangup after socket close' );
+      }
+
+      my ( $Prd, $Pwr ) = IO::Async::OS->pipepair or die "Cannot pipepair - $!";
+      $_->blocking( 0 ) for $Prd, $Pwr;
 
       my $hangup = 0;
-      $loop->watch_io(
-         handle => $S1,
-         on_hangup => sub { $hangup = 1 },
-      );
-
-      $S2->close;
-
-      $loop->loop_once( 0.1 );
-
-      is( $hangup, 1, '$hangup after socket close' );
-
-      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
-      $_->blocking( 0 ) for $Prd, $Pwr;
-
-      $hangup = 0;
-      $loop->watch_io(
-         handle => $Prd,
-         on_hangup => sub { $hangup = 1 },
-      );
-
-      $Pwr->close;
-
-      $loop->loop_once( 0.1 );
-
-      is( $hangup, 1, '$hangup after pipe close for reading' );
-
-      ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
-      $_->blocking( 0 ) for $Prd, $Pwr;
-
-      $hangup = 0;
       $loop->watch_io(
          handle => $Pwr,
          on_hangup => sub { $hangup = 1 },
@@ -299,7 +293,7 @@ sub run_tests_io
 
    # Check that combined read/write handlers can cancel each other
    {
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot socketpair - $!";
       $_->blocking( 0 ) for $S1, $S2;
 
       my $callcount = 0;
@@ -325,7 +319,7 @@ sub run_tests_io
    # Check that error conditions that aren't true read/write-ability are still
    # invoked
    {
-      my ( $S1, $S2 ) = $loop->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
       $_->blocking( 0 ) for $S1, $S2;
       $S2->close;
 
@@ -381,10 +375,78 @@ Tests the Loop's ability to handle timer events
 
 =cut
 
-use constant count_tests_timer => 12;
+use constant count_tests_timer => 22;
 sub run_tests_timer
 {
    my $done = 0;
+   # New watch/unwatch API
+
+   cmp_ok( abs( $loop->time - time ), "<", 0.1, '$loop->time gives the current time' );
+
+   $loop->watch_time( after => 2 * AUT, code => sub { $done = 1; } );
+
+   is_oneref( $loop, '$loop has refcount 1 after watch_time' );
+
+   time_between {
+      my $now = time;
+      $loop->loop_once( 5 * AUT );
+
+      # poll might have returned just a little early, such that the TimerQueue
+      # doesn't think anything is ready yet. We need to handle that case.
+      while( !$done ) {
+         die "It should have been ready by now" if( time - $now > 5 * AUT );
+         $loop->loop_once( 0.1 * AUT );
+      }
+   } 1.5, 2.5, 'loop_once(5) while waiting for watch_time after';
+
+   $loop->watch_time( at => time + 2 * AUT, code => sub { $done = 2; } );
+
+   time_between {
+      my $now = time;
+      $loop->loop_once( 5 * AUT );
+
+      # poll might have returned just a little early, such that the TimerQueue
+      # doesn't think anything is ready yet. We need to handle that case.
+      while( !$done ) {
+         die "It should have been ready by now" if( time - $now > 5 * AUT );
+         $loop->loop_once( 0.1 * AUT );
+      }
+   } 1.5, 2.5, 'loop_once(5) while waiting for watch_time at';
+
+   my $cancelled_fired = 0;
+   my $id = $loop->watch_time( after => 1 * AUT, code => sub { $cancelled_fired = 1 } );
+   $loop->unwatch_time( $id );
+   undef $id;
+
+   $loop->loop_once( 2 * AUT );
+
+   ok( !$cancelled_fired, 'unwatched watch_time does not fire' );
+
+   $loop->watch_time( after => -1, code => sub { $done = 1 } );
+
+   $done = 0;
+
+   time_between {
+      $loop->loop_once while !$done;
+   } 0, 0.1, 'loop_once while waiting for negative interval timer';
+
+   my $doneA;
+   my $doneB;
+
+   $id = $loop->watch_time( after => 1 * AUT, code => sub {
+      $loop->unwatch_time( $id ); undef $id;
+      $doneA++;
+   });
+
+   $loop->watch_time( after => 1.1 * AUT, code => sub { $doneB++ } );
+
+   $loop->loop_once( 1 * AUT ) for 1 .. 3;
+
+   is( $doneA, 1, 'Self-cancelling timer still fires' );
+   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
+
+   # Legacy enqueue/requeue/cancel API
+   $done = 0;
 
    $loop->enqueue_timer( delay => 2 * AUT, code => sub { $done = 1; } );
 
@@ -402,8 +464,25 @@ sub run_tests_timer
       }
    } 1.5, 2.5, 'loop_once(5) while waiting for timer';
 
-   my $cancelled_fired = 0;
-   my $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub { $cancelled_fired = 1 } );
+   # Check that short delays are achievable in one ->loop_once call
+   foreach my $delay ( 0.001, 0.01, 0.1 ) {
+      my $done;
+      my $count = 0;
+      my $start = time;
+
+      $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+
+      while( !$done ) {
+         $loop->loop_once( 1 );
+         $count++;
+         last if time - $start > 5; # bailout
+      }
+
+      is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
+   }
+
+   $cancelled_fired = 0;
+   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub { $cancelled_fired = 1 } );
    $loop->cancel_timer( $id );
    undef $id;
 
@@ -433,29 +512,6 @@ sub run_tests_timer
    } 1.5, 2.5, 'requeued timer of delay 2';
 
    is( $done, 2, '$done is 2 after requeued timer' );
-
-   $loop->enqueue_timer( delay => -1, code => sub { $done = 1 } );
-
-   $done = 0;
-
-   time_between {
-      $loop->loop_once while !$done;
-   } 0, 0.1, 'loop_once while waiting for negative interval timer';
-
-   my $doneA;
-   my $doneB;
-
-   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub {
-      $loop->cancel_timer( $id ); undef $id;
-      $doneA++;
-   });
-
-   $loop->enqueue_timer( delay => 1.1 * AUT, code => sub { $doneB++ } );
-
-   $loop->loop_once( 1 * AUT ) for 1 .. 3;
-
-   is( $doneA, 1, 'Self-cancelling timer still fires' );
-   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
 }
 
 =head2 signal
@@ -537,7 +593,7 @@ Tests the Loop's support for idle handlers
 
 =cut
 
-use constant count_tests_idle => 10;
+use constant count_tests_idle => 11;
 sub run_tests_idle
 {
    my $called = 0;
@@ -574,11 +630,20 @@ sub run_tests_idle
 
    is( $called, 2, 'unwatched deferral not called' );
 
-   $loop->later( sub { $called = 3 } );
+   $id = $loop->watch_idle( when => 'later', code => sub { $called = 3 } );
+   my $timer_id = $loop->watch_time( after => 5, code => sub {} );
 
    $loop->loop_once( 1 );
 
-   is( $called, 3, '$loop->later shortcut works' );
+   is( $called, 3, '$loop->later still invoked with enqueued timer' );
+
+   $loop->unwatch_time( $timer_id );
+
+   $loop->later( sub { $called = 4 } );
+
+   $loop->loop_once( 1 );
+
+   is( $called, 4, '$loop->later shortcut works' );
 }
 
 =head2 child
@@ -652,18 +717,46 @@ sub run_tests_child
 
 =head2 control
 
-Tests that the C<loop_once> and C<loop_forever> methods behave correctly
+Tests that the C<run>, C<stop>, C<loop_once> and C<loop_forever> methods
+behave correctly
 
 =cut
 
-use constant count_tests_control => 5;
+use constant count_tests_control => 8;
 sub run_tests_control
 {
    time_between { $loop->loop_once( 0 ) } 0, 0.1, 'loop_once(0) when idle';
 
    time_between { $loop->loop_once( 2 * AUT ) } 1.5, 2.5, 'loop_once(2) when idle';
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->loop_stop } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->stop( result => "here" ) } );
+
+   local $SIG{ALRM} = sub { die "Test timed out before ->stop" };
+   alarm( 1 );
+
+   my @result = $loop->run;
+
+   alarm( 0 );
+
+   is_deeply( \@result, [ result => "here" ], '->stop arguments returned by ->run' );
+
+   $loop->watch_time( after => 0.1, code => sub { $loop->stop( result => "here" ) } );
+
+   my $result = $loop->run;
+
+   is( $result, "result", 'First ->stop argument returned by ->run in scalar context' );
+
+   $loop->watch_time( after => 0.1, code => sub {
+      $loop->watch_time( after => 0.1, code => sub { $loop->stop( "inner" ) } );
+      my @result = $loop->run;
+      $loop->stop( @result, "outer" );
+   } );
+
+   @result = $loop->run;
+
+   is_deeply( \@result, [ "inner", "outer" ], '->run can be nested properly' );
+
+   $loop->watch_time( after => 0.1, code => sub { $loop->loop_stop } );
 
    local $SIG{ALRM} = sub { die "Test timed out before ->loop_stop" };
    alarm( 1 );
