@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::Test;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.42';
+our $VERSION = '0.51';
 
 use Exporter 'import';
 our @EXPORT = qw(
@@ -27,7 +27,7 @@ C<IO::Async::Test> - utility functions for use in test scripts
  use IO::Async::Test;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
  testing_loop( $loop );
 
  my $result;
@@ -64,14 +64,14 @@ actions to perform, interspersed with assertions which check for given
 conditions. This goes against the very nature of C<IO::Async> which, being an
 asynchronisation framework, does not provide a linear stepped way of working.
 
-In order to write a test, the C<wait_for()> function provides a way of
+In order to write a test, the C<wait_for> function provides a way of
 synchronising the code, so that a given condition is known to hold, which
 would typically signify that some event has occured, the outcome of which can
 now be tested using the usual testing primitives.
 
 Because the primary purpose of C<IO::Async> is to provide IO operations on
 filehandles, a great many tests will likely be based around connected pipes or
-socket handles. The C<wait_for_stream()> function provides a convenient way
+socket handles. The C<wait_for_stream> function provides a convenient way
 to wait for some content to be written through such a connected stream.
 
 =cut
@@ -84,7 +84,7 @@ my $loop;
 
 =head2 testing_loop( $loop )
 
-Set the C<IO::Async::Loop> object which the C<wait_for()> function will loop
+Set the C<IO::Async::Loop> object which the C<wait_for> function will loop
 on.
 
 =cut
@@ -96,8 +96,8 @@ sub testing_loop
 
 =head2 wait_for( $condfunc )
 
-Repeatedly call the C<loop_once()> method on the underlying loop (given to the
-C<testing_loop()> function), until the given condition function callback
+Repeatedly call the C<loop_once> method on the underlying loop (given to the
+C<testing_loop> function), until the given condition function callback
 returns true.
 
 To guard against stalled scripts, if the loop indicates a timeout for 10
@@ -109,11 +109,11 @@ sub wait_for(&)
 {
    my ( $cond ) = @_;
 
-   my ( undef, $callerfile, $callerline ) = caller();
+   my ( undef, $callerfile, $callerline ) = caller;
 
    my $timedout = 0;
-   my $timerid = $loop->enqueue_timer(
-      delay => 10,
+   my $timerid = $loop->watch_time(
+      after => 10,
       code => sub { $timedout = 1 },
    );
 
@@ -123,7 +123,7 @@ sub wait_for(&)
       die "Nothing was ready after 10 second wait; called at $callerfile line $callerline\n";
    }
    else {
-      $loop->cancel_timer( $timerid );
+      $loop->unwatch_time( $timerid );
    }
 }
 
