@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::Timer;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.42';
+our $VERSION = '0.51';
 
 use Carp;
 
@@ -19,7 +19,7 @@ C<IO::Async::Timer> - base class for Notifiers that use timed delays
 
 =head1 DESCRIPTION
 
-This module provides a base class of L<IO::Async::Notifier> for implementing
+This module provides a subclass of L<IO::Async::Notifier> for implementing
 notifiers that use timed delays. For specific implementations, see one of the
 subclasses:
 
@@ -131,7 +131,7 @@ sub start
 {
    my $self = shift;
 
-   my $loop = $self->get_loop;
+   my $loop = $self->loop;
    if( !defined $loop ) {
       $self->{pending} = 1;
       return $self;
@@ -143,7 +143,7 @@ sub start
       $self->{cb} = $self->_make_cb;
    }
 
-   $self->{id} = $loop->enqueue_timer(
+   $self->{id} = $loop->watch_time(
       $self->_make_enqueueargs,
       code => $self->{cb},
    );
@@ -167,11 +167,13 @@ sub stop
       return;
    }
 
-   my $loop = $self->get_loop or croak "Cannot stop a Timer that is not in a Loop";
+   return if !$self->is_running;
+
+   my $loop = $self->loop or croak "Cannot stop a Timer that is not in a Loop";
 
    defined $self->{id} or return; # nothing to do but no error
 
-   $loop->cancel_timer( $self->{id} );
+   $loop->unwatch_time( $self->{id} );
 
    undef $self->{id};
 }
